//  Copyright 2016 root.ext@gmail.com
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import 'dart:typed_data';
import '../lib/dna.dart';

const int VK_SUCCESS = 0;

int VK_STRUCTURE_TYPE_APPLICATION_INFO = 0;
int VK_STRUCTURE_TYPE_INSTANCE_CREATE_INFO = 1;

@Library('libvulkan.so.1', 'vulkan-1.dll')
class Vulkan {
  dynamic lib = new DynamicLibrary<Vulkan>();

  int vkCreateInstance(
    @Param(C_STRUCT) VkInstanceCreateInfo createInfo,
    @Param(C_POINTER) int allocator,
    @Param(TYPEDDATA) TypedData instance)
  { return lib.vkCreateInstance(createInfo, allocator, instance); }


  int vkEnumeratePhysicalDevices(
    @Param(C_POINTER) int instance,
    @Param(TYPEDDATA) TypedData pPhysicalDeviceCount,
    @Param(C_POINTER) int pPhysicalDevices)
  { return lib.vkEnumeratePhysicalDevices(instance, pPhysicalDeviceCount, pPhysicalDevices); }

}

@Struct()
class VkApplicationInfo {
  @Field(C_INT) int sType;
  @Field(C_POINTER) int pNext;
  @Field(C_STRING) String pApplicationName;
  @Field(C_UINT) int applicationVersion;
  @Field(C_STRING) String pEngineName;
  @Field(C_UINT) int engineVersion;
  @Field(C_UINT) int apiVersion;
}

@Struct()
class VkInstanceCreateInfo {
  @Field(C_INT)  int sType;
  @Field(C_POINTER) int pNext;
  @Field(C_UINT) int flags;
  @Field(C_STRUCT) VkApplicationInfo pApplicationInfo;
  @Field(C_UINT) int enabledLayerCount;
  @Field(C_POINTER) List<String> ppEnabledLayerNames;
  @Field(C_UINT) int enabledExtensionCount;
  @Field(C_POINTER) List<String> ppEnabledExtensionNames;
}


class VkAllocationCallbacks {}
class VkInstance {}